package fc.SimpleRaytracer;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import fc.SimpleRaytracer.Geometry.Thing;
import fc.SimpleRaytracer.Math.Vector;
import fc.SimpleRaytracer.Rendering.Camera;
import fc.SimpleRaytracer.Rendering.Color;
import fc.SimpleRaytracer.Rendering.Light;
import fc.SimpleRaytracer.Rendering.Raytracer;
import fc.SimpleRaytracer.Rendering.Scene;

public class MainApplication
{
	public static void main(String[] args)
	{
		new MainApplication().run(args);
	}
	
	public void run(String[] args)
	{
		BufferedImage bi = new BufferedImage(800, 800, BufferedImage.TYPE_INT_RGB);
		
		double k = 2.0;
		double y = 0.2;
		Thing[] things =
			{
				//new Plane(new Vector(0.0, 1.0, 0.0), 0.0, new Checkerboard()),
				//new Sphere(new Vector(0.0, 1.0, -0.25), 1.0, new ShinyMaterial()),
				//new Sphere(new Vector(-1.0, 0.5, 0.0), 0.25, new ShinySurface()),
				//new Sphere(new Vector(-1.0, 0.5, 1.5), 0.5, new ShinyMaterial()),
				//new Triangle(new Vector(-k,y,-k), new Vector(-k, y, k), new Vector(k, y, k), new ShinySurface()),
				//new Triangle(new Vector(-k,y,-k), new Vector( k, y, k), new Vector(k, y,-k), new ShinySurface())
			};
		
		Light[] lights =
			{
				new Light(new Vector(-2.0, 2.5, 0.0), new Color(0.49, 0.07, 0.07)),
				new Light(new Vector( 1.5, 2.5, 1.5), new Color(0.07, 0.07, 0.49)),
				new Light(new Vector( 1.5, 2.5,-1.5), new Color(0.07, 0.49, 0.071)),
				new Light(new Vector( 0.0, 3.5, 0.0), new Color(0.21, 0.21, 0.35)),
			};
		
		Camera camera = new Camera(new Vector(3.0, 2.0, 4.0), new Vector(-1.0, 0.5, 0.0));
		
		Scene scene = new Scene(things, lights, camera);
		
		try
		{
			Raytracer raytracer = new Raytracer();
			raytracer.render(scene, bi);
		}
		catch (Exception e)
		{
			System.out.println(e.toString());
		}
	
	    File outputfile = new File("result.png");
	    
	    try
	    {
	    	ImageIO.write(bi, "png", outputfile);
	    }
	    catch (IOException e)
	    {
	    	System.out.println("Error, IOException caught: " + e.toString());
	    }
	}
}
